@@ -4,12 +4,11 @@
  +-------------------------------------------------------------------------+
  | GnuPG (PGP) driver for the Enigma Plugin                                |
  |                                                                         |
- | Copyright (C) 2010-2015 The Roundcube Dev Team                          |
+ | Copyright (C) The Roundcube Dev Team                                    |
  |                                                                         |
  | Licensed under the GNU General Public License version 3 or              |
  | any later version with exceptions for skins & plugins.                  |
  | See the README file for a full license statement.                       |
- |                                                                         |
  +-------------------------------------------------------------------------+
  | Author: Aleksander Machniak <alec@alec.pl>                              |
  +-------------------------------------------------------------------------+
@@ -24,8 +23,15 @@ class enigma_driver_gnupg extends enigma_driver
     protected $homedir;
     protected $user;
     protected $last_sig_algorithm;
+    protected $debug    = false;
+    protected $db_files = ['pubring.gpg', 'secring.gpg', 'pubring.kbx'];
 
 
+    /**
+     * Class constructor
+     *
+     * @param rcube_user $user User object
+     */
     function __construct($user)
     {
         $this->rc   = rcmail::get_instance();
@@ -36,11 +42,11 @@ class enigma_driver_gnupg extends enigma_driver
      * Driver initialization and environment checking.
      * Should only return critical errors.
      *
-     * @return mixed NULL on success, enigma_error on failure
+     * @return enigma_error|null NULL on success, enigma_error on failure
      */
     function init()
     {
-        $homedir = $this->rc->config->get('enigma_pgp_homedir', INSTALL_PATH . 'plugins/enigma/home');
+        $homedir = $this->rc->config->get('enigma_pgp_homedir');
         $debug   = $this->rc->config->get('enigma_debug');
         $binary  = $this->rc->config->get('enigma_pgp_binary');
         $agent   = $this->rc->config->get('enigma_pgp_agent');
@@ -77,12 +83,13 @@ class enigma_driver_gnupg extends enigma_driver
                 "Unable to write to keys directory: $homedir");
         }
 
+        $this->debug   = $debug;
         $this->homedir = $homedir;
 
-        $options = array('homedir' => $this->homedir);
+        $options = ['homedir' => $this->homedir];
 
         if ($debug) {
-            $options['debug'] = array($this, 'debug');
+            $options['debug'] = [$this, 'debug'];
         }
         if ($binary) {
             $options['binary'] = $binary;
@@ -94,6 +101,9 @@ class enigma_driver_gnupg extends enigma_driver
             $options['gpgconf'] = $gpgconf;
         }
 
+        $options['cipher-algo'] = $this->rc->config->get('enigma_pgp_cipher_algo');
+        $options['digest-algo'] = $this->rc->config->get('enigma_pgp_digest_algo');
+
         // Create Crypt_GPG object
         try {
             $this->gpg = new Crypt_GPG($options);
@@ -101,16 +111,18 @@ class enigma_driver_gnupg extends enigma_driver
         catch (Exception $e) {
             return $this->get_error_from_exception($e);
         }
+
+        $this->db_sync();
     }
 
     /**
      * Encryption (and optional signing).
      *
-     * @param string     Message body
-     * @param array      List of keys (enigma_key objects)
-     * @param enigma_key Optional signing Key ID
+     * @param string     $text     Message body
+     * @param array      $keys     List of keys (enigma_key objects)
+     * @param enigma_key $sign_key Optional signing Key ID
      *
-     * @return mixed Encrypted message or enigma_error on failure
+     * @return string|enigma_error Encrypted message or enigma_error on failure
      */
     function encrypt($text, $keys, $sign_key = null)
     {
@@ -140,13 +152,13 @@ class enigma_driver_gnupg extends enigma_driver
     /**
      * Decrypt a message (and verify if signature found)
      *
-     * @param string           Encrypted message
-     * @param array            List of key-password mapping
-     * @param enigma_signature Signature information (if available)
+     * @param string           $text       Encrypted message
+     * @param array            $keys       List of key-password mapping
+     * @param enigma_signature &$signature Signature information (if available)
      *
      * @return mixed Decrypted message or enigma_error on failure
      */
-    function decrypt($text, $keys = array(), &$signature = null)
+    function decrypt($text, $keys = [], &$signature = null)
     {
         try {
             foreach ($keys as $key => $password) {
@@ -179,9 +191,9 @@ class enigma_driver_gnupg extends enigma_driver
     /**
      * Signing.
      *
-     * @param string     Message body
-     * @param enigma_key The key
-     * @param int        Signing mode (enigma_engine::SIGN_*)
+     * @param string     $text Message body
+     * @param enigma_key $key  The key
+     * @param int        $mode Signing mode (enigma_engine::SIGN_*)
      *
      * @return mixed True on success or enigma_error on failure
      */
@@ -190,7 +202,7 @@ class enigma_driver_gnupg extends enigma_driver
         try {
             $this->gpg->addSignKey($key->reference, $key->password);
 
-            $res     = $this->gpg->sign($text, $mode, CRYPT_GPG::ARMOR_ASCII, true);
+            $res     = $this->gpg->sign($text, $mode, Crypt_GPG::ARMOR_ASCII, true);
             $sigInfo = $this->gpg->getLastSignatureInfo();
 
             $this->last_sig_algorithm = $sigInfo->getHashAlgorithmName();
@@ -205,10 +217,10 @@ class enigma_driver_gnupg extends enigma_driver
     /**
      * Signature verification.
      *
-     * @param string Message body
-     * @param string Signature, if message is of type PGP/MIME and body doesn't contain it
+     * @param string $text      Message body
+     * @param string $signature Signature, if message is of type PGP/MIME and body doesn't contain it
      *
-     * @return mixed Signature information (enigma_signature) or enigma_error
+     * @return enigma_signature|enigma_error Signature information or enigma_error
      */
     function verify($text, $signature)
     {
@@ -224,13 +236,13 @@ class enigma_driver_gnupg extends enigma_driver
     /**
      * Key file import.
      *
-     * @param string File name or file content
-     * @param bolean True if first argument is a filename
-     * @param array  Optional key => password map
+     * @param string $content   File name or file content
+     * @param bool   $isfile    True if first argument is a filename
+     * @param array  $passwords Optional key => password map
      *
      * @return mixed Import status array or enigma_error
      */
-    public function import($content, $isfile = false, $passwords = array())
+    public function import($content, $isfile = false, $passwords = [])
     {
         try {
             // GnuPG 2.1 requires secret key passphrases on import
@@ -238,10 +250,16 @@ class enigma_driver_gnupg extends enigma_driver
                 $this->gpg->addPassphrase($keyid, $pass);
             }
 
-            if ($isfile)
-                return $this->gpg->importKeyFile($content);
-            else
-                return $this->gpg->importKey($content);
+            if ($isfile) {
+                $result = $this->gpg->importKeyFile($content);
+            }
+            else {
+                $result = $this->gpg->importKey($content);
+            }
+
+            $this->db_save();
+
+            return $result;
         }
         catch (Exception $e) {
             return $this->get_error_from_exception($e);
@@ -251,13 +269,13 @@ class enigma_driver_gnupg extends enigma_driver
     /**
      * Key export.
      *
-     * @param string Key ID
-     * @param bool   Include private key
-     * @param array  Optional key => password map
+     * @param string $keyid        Key ID
+     * @param bool   $with_private Include private key
+     * @param array  $passwords    Optional key => password map
      *
-     * @return mixed Key content or enigma_error
+     * @return string|enigma_error Key content or enigma_error
      */
-    public function export($keyid, $with_private = false, $passwords = array())
+    public function export($keyid, $with_private = false, $passwords = [])
     {
         try {
             $key = $this->gpg->exportPublicKey($keyid, true);
@@ -282,19 +300,18 @@ class enigma_driver_gnupg extends enigma_driver
     /**
      * Keys listing.
      *
-     * @param string Optional pattern for key ID, user ID or fingerprint
+     * @param string $pattern Optional pattern for key ID, user ID or fingerprint
      *
-     * @return mixed Array of enigma_key objects or enigma_error
+     * @return enigma_key[]|enigma_error Array of keys or enigma_error
      */
     public function list_keys($pattern = '')
     {
         try {
-            $keys = $this->gpg->getKeys($pattern);
-            $result = array();
+            $keys   = $this->gpg->getKeys($pattern);
+            $result = [];
 
             foreach ($keys as $idx => $key) {
                 $result[] = $this->parse_key($key);
-                unset($keys[$idx]);
             }
 
             return $result;
@@ -307,16 +324,16 @@ class enigma_driver_gnupg extends enigma_driver
     /**
      * Single key information.
      *
-     * @param string Key ID, user ID or fingerprint
+     * @param string $keyid Key ID, user ID or fingerprint
      *
-     * @return mixed Key (enigma_key) object or enigma_error
+     * @return enigma_key|enigma_error Key object or enigma_error
      */
     public function get_key($keyid)
     {
         $list = $this->list_keys($keyid);
 
         if (is_array($list)) {
-            return $list[key($list)];
+            return array_first($list);
         }
 
         // error
@@ -326,7 +343,7 @@ class enigma_driver_gnupg extends enigma_driver
     /**
      * Key pair generation.
      *
-     * @param array Key/User data (user, email, password, size)
+     * @param array $data Key/User data (user, email, password, size)
      *
      * @return mixed Key (enigma_key) object or enigma_error
      */
@@ -334,11 +351,11 @@ class enigma_driver_gnupg extends enigma_driver
     {
         try {
             $debug  = $this->rc->config->get('enigma_debug');
-            $keygen = new Crypt_GPG_KeyGenerator(array(
+            $keygen = new Crypt_GPG_KeyGenerator([
                     'homedir' => $this->homedir,
                     // 'binary'  => '/usr/bin/gpg2',
-                    'debug'   => $debug ? array($this, 'debug') : false,
-            ));
+                    'debug'   => $debug ? [$this, 'debug'] : false,
+            ]);
 
             $key = $keygen
                 ->setExpirationDate(0)
@@ -355,7 +372,7 @@ class enigma_driver_gnupg extends enigma_driver
     /**
      * Key deletion.
      *
-     * @param string Key ID
+     * @param string $keyid Key ID
      *
      * @return mixed True on success or enigma_error
      */
@@ -382,6 +399,8 @@ class enigma_driver_gnupg extends enigma_driver
             }
         }
 
+        $this->db_save();
+
         return $result;
     }
 
@@ -396,6 +415,23 @@ class enigma_driver_gnupg extends enigma_driver
         return $this->last_sig_algorithm;
     }
 
+    /**
+     * Returns a list of supported features.
+     *
+     * @return array Capabilities list
+     */
+    public function capabilities()
+    {
+        $caps = [enigma_driver::SUPPORT_RSA];
+        $version = $this->gpg->getVersion();
+
+        if (version_compare($version, '2.1.7', 'ge')) {
+            $caps[] = enigma_driver::SUPPORT_ECC;
+        }
+
+        return $caps;
+    }
+
     /**
      * Private key deletion.
      */
@@ -427,13 +463,13 @@ class enigma_driver_gnupg extends enigma_driver
     /**
      * Converts Crypt_GPG exception into Enigma's error object
      *
-     * @param mixed Exception object
+     * @param mixed $e Exception object
      *
      * @return enigma_error Error object
      */
     protected function get_error_from_exception($e)
     {
-        $data = array();
+        $data = [];
 
         if ($e instanceof Crypt_GPG_KeyNotFoundException) {
             $error = enigma_error::KEYNOTFOUND;
@@ -462,7 +498,7 @@ class enigma_driver_gnupg extends enigma_driver
     /**
      * Converts Crypt_GPG_Signature object into Enigma's signature object
      *
-     * @param Crypt_GPG_Signature Signature object
+     * @param Crypt_GPG_Signature $sig Signature object
      *
      * @return enigma_signature Signature object
      */
@@ -489,7 +525,7 @@ class enigma_driver_gnupg extends enigma_driver
     /**
      * Converts Crypt_GPG_Key object into Enigma's key object
      *
-     * @param Crypt_GPG_Key Key object
+     * @param Crypt_GPG_Key $key Key object
      *
      * @return enigma_key Key object
      */
@@ -517,14 +553,29 @@ class enigma_driver_gnupg extends enigma_driver
             $skey = new enigma_subkey();
             $skey->id          = $subkey->getId();
             $skey->revoked     = $subkey->isRevoked();
-            $skey->created     = $subkey->getCreationDate();
-            $skey->expires     = $subkey->getExpirationDate();
             $skey->fingerprint = $subkey->getFingerprint();
             $skey->has_private = $subkey->hasPrivate();
             $skey->algorithm   = $subkey->getAlgorithm();
             $skey->length      = $subkey->getLength();
             $skey->usage       = $subkey->usage();
 
+            if (method_exists($subkey, 'getCreationDateTime')) {
+                $skey->created = $subkey->getCreationDateTime();
+                $skey->expires = $subkey->getExpirationDateTime();
+            }
+            else {
+                $skey->created = $subkey->getCreationDate();
+                $skey->expires = $subkey->getExpirationDate();
+
+                if ($skey->created) {
+                    $skey->created = new DateTime("@{$skey->created}");
+                }
+
+                if ($skey->expires) {
+                    $skey->expires = new DateTime("@{$skey->expires}");
+                }
+            }
+
             $ekey->subkeys[$idx] = $skey;
         };
 
@@ -533,6 +584,203 @@ class enigma_driver_gnupg extends enigma_driver
         return $ekey;
     }
 
+    /**
+     * Synchronize keys database on multi-host setups
+     */
+    protected function db_sync()
+    {
+        if (!$this->rc->config->get('enigma_multihost')) {
+            return;
+        }
+
+        $db    = $this->rc->get_dbh();
+        $table = $db->table_name('filestore', true);
+        $files = [];
+
+        $result = $db->query(
+            "SELECT `file_id`, `filename`, `mtime` FROM $table WHERE `user_id` = ? AND `context` = ?",
+            $this->rc->user->ID, 'enigma'
+        );
+
+        while ($record = $db->fetch_assoc($result)) {
+            $file  = $this->homedir . '/' . $record['filename'];
+            $mtime = @filemtime($file);
+            $files[] = $record['filename'];
+
+            if ($mtime < $record['mtime']) {
+                $data_result = $db->query("SELECT `data`, `mtime` FROM $table"
+                    . " WHERE `file_id` = ?", $record['file_id']
+                );
+
+                $record = $db->fetch_assoc($data_result);
+                $data   = $record ? base64_decode($record['data']) : null;
+
+                if ($data === null || $data === false) {
+                    rcube::raise_error([
+                            'code' => 605, 'line' => __LINE__, 'file' => __FILE__,
+                            'message' => "Enigma: Failed to sync $file ({$record['file_id']}). Decode error."
+                        ], true, false);
+
+                    continue;
+                }
+
+                // Private keys might be located in 'private-keys-v1.d' subdirectory. Make sure it exists.
+                if (strpos($file, '/private-keys-v1.d/')) {
+                    if (!file_exists($this->homedir . '/private-keys-v1.d')) {
+                        mkdir($this->homedir . '/private-keys-v1.d', 0700);
+                    }
+                }
+
+                $tmpfile = $file . '.tmp';
+
+                if (file_put_contents($tmpfile, $data, LOCK_EX) === strlen($data)) {
+                    rename($tmpfile, $file);
+                    touch($file, $record['mtime']);
+
+                    if ($this->debug) {
+                        $this->debug("SYNC: Fetched file: $file");
+                    }
+                }
+                else {
+                    // error
+                    @unlink($tmpfile);
+
+                    rcube::raise_error([
+                            'code' => 605, 'line' => __LINE__, 'file' => __FILE__,
+                            'message' => "Enigma: Failed to sync $file."
+                        ], true, false);
+                }
+            }
+        }
+
+        // Remove files not in database
+        if (!$db->is_error($result)) {
+            foreach (array_diff($this->db_files_list(), $files) as $file) {
+                $file = $this->homedir . '/' . $file;
+
+                if (unlink($file)) {
+                    if ($this->debug) {
+                        $this->debug("SYNC: Removed file: $file");
+                    }
+                }
+            }
+        }
+
+        // No records found, do initial sync if already have the keyring
+        if (!$db->is_error($result) && empty($file)) {
+            $this->db_save(true);
+        }
+    }
+
+    /**
+     * Save keys database for multi-host setups
+     */
+    protected function db_save($is_empty = false)
+    {
+        if (!$this->rc->config->get('enigma_multihost')) {
+            return true;
+        }
+
+        $db      = $this->rc->get_dbh();
+        $table   = $db->table_name('filestore', true);
+        $records = [];
+
+        if (!$is_empty) {
+            $result = $db->query(
+                "SELECT `file_id`, `filename`, `mtime` FROM $table WHERE `user_id` = ? AND `context` = ?",
+                $this->rc->user->ID, 'enigma'
+            );
+
+            while ($record = $db->fetch_assoc($result)) {
+                $records[$record['filename']] = $record;
+            }
+        }
+
+        foreach ($this->db_files_list() as $filename) {
+            $file  = $this->homedir . '/' . $filename;
+            $mtime = @filemtime($file);
+
+            $existing = !empty($records[$filename]) ? $records[$filename] : null;
+            unset($records[$filename]);
+
+            if ($mtime && (empty($existing) || $mtime > $existing['mtime'])) {
+                $data     = file_get_contents($file);
+                $data     = base64_encode($data);
+                $datasize = strlen($data);
+
+                if (empty($maxsize)) {
+                    $maxsize = min($db->get_variable('max_allowed_packet', 1048500), 4*1024*1024) - 2000;
+                }
+
+                if ($datasize > $maxsize) {
+                    rcube::raise_error([
+                            'code' => 605, 'line' => __LINE__, 'file' => __FILE__,
+                            'message' => "Enigma: Failed to save $file. Size exceeds max_allowed_packet."
+                        ], true, false);
+
+                    continue;
+                }
+
+                $unique = ['user_id' => $this->rc->user->ID, 'context' => 'enigma', 'filename' => $filename];
+                $result = $db->insert_or_update($table, $unique, ['mtime', 'data'], [$mtime, $data]);
+
+                if ($db->is_error($result)) {
+                    rcube::raise_error([
+                            'code' => 605, 'line' => __LINE__, 'file' => __FILE__,
+                            'message' => "Enigma: Failed to save $file into database."
+                        ], true, false);
+
+                    break;
+                }
+
+                if ($this->debug) {
+                    $this->debug("SYNC: Pushed file: $file");
+                }
+            }
+        }
+
+        // Delete removed files from database
+        foreach (array_keys($records) as $filename) {
+            $file   = $this->homedir . '/' . $filename;
+            $result = $db->query("DELETE FROM $table WHERE `user_id` = ? AND `context` = ? AND `filename` = ?",
+                $this->rc->user->ID, 'enigma', $filename
+            );
+
+            if ($db->is_error($result)) {
+                rcube::raise_error([
+                        'code' => 605, 'line' => __LINE__, 'file' => __FILE__,
+                        'message' => "Enigma: Failed to delete $file from database."
+                    ], true, false);
+
+                break;
+            }
+
+            if ($this->debug) {
+                $this->debug("SYNC: Removed file: $file");
+            }
+        }
+    }
+
+    /**
+     * Returns list of homedir files to backup
+     */
+    protected function db_files_list()
+    {
+        $files = [];
+
+        foreach ($this->db_files as $file) {
+            if (file_exists($this->homedir . '/' . $file)) {
+                $files[] = $file;
+            }
+        }
+
+        foreach (glob($this->homedir . '/private-keys-v1.d/*.key') as $file) {
+            $files[] = ltrim(substr($file, strlen($this->homedir)), '/');
+        }
+
+        return $files;
+    }
+
     /**
      * Write debug info from Crypt_GPG to logs/enigma
      */
