@@ -45,8 +45,9 @@ class rcmail_action_mail_mark extends rcmail_action_mail_index
         $read_deleted = (bool) $rcmail->config->get('read_when_deleted');
         $flag         = self::imap_flag($flag);
         $old_count    = 0;
+        $from         = $_POST['_from'] ?? null;
 
-        if ($flag == 'DELETED' && $skip_deleted && (!isset($_POST['_from']) || $_POST['_from'] != 'show')) {
+        if ($flag == 'DELETED' && $skip_deleted && $from != 'show') {
             // count messages before changing anything
             $old_count = $rcmail->storage->count(null, $threading ? 'THREADS' : 'ALL');
         }
@@ -79,7 +80,7 @@ class rcmail_action_mail_mark extends rcmail_action_mail_index
 
         if (!$marked) {
             // send error message
-            if (empty($_POST['_from']) || $_POST['_from'] != 'show') {
+            if ($from != 'show') {
                 $rcmail->output->command('list_mailbox');
             }
 
@@ -110,7 +111,7 @@ class rcmail_action_mail_mark extends rcmail_action_mail_index
             $rcmail->output->set_env('last_flag', $flag);
         }
         else if ($flag == 'DELETED' && $skip_deleted) {
-            if ($_POST['_from'] == 'show') {
+            if ($from == 'show') {
                 if ($next = rcube_utils::get_input_value('_next_uid', rcube_utils::INPUT_GPC)) {
                     $rcmail->output->command('show_message', $next);
                 }
