@@ -75,8 +75,8 @@ class rcmail extends rcube
     /**
      * This implements the 'singleton' design pattern
      *
-     * @param integer $mode Ignored rcube::get_instance() argument
-     * @param string  $env  Environment name to run (e.g. live, dev, test)
+     * @param int    $mode Ignored rcube::get_instance() argument
+     * @param string $env  Environment name to run (e.g. live, dev, test)
      *
      * @return rcmail The one and only instance
      */
@@ -211,7 +211,7 @@ class rcmail extends rcube
         // set localization
         setlocale(LC_ALL, $lang . '.utf8', $lang . '.UTF-8', 'en_US.utf8', 'en_US.UTF-8');
 
-        // Workaround for http://bugs.php.net/bug.php?id=18556
+        // Workaround for https://bugs.php.net/bug.php?id=18556
         // Also strtoupper/strtolower and other methods are locale-aware
         // for these locales it is problematic (#1490519)
         if (in_array($lang, ['tr_TR', 'ku', 'az_AZ'])) {
@@ -968,7 +968,7 @@ class rcmail extends rcube
             list(, $domain) = rcube_utils::explode('@', $post_user);
 
             // direct match in default_host array
-            if ($default_host[$post_host] || in_array($post_host, array_values($default_host))) {
+            if (!empty($default_host[$post_host]) || in_array($post_host, array_values($default_host))) {
                 $host = $post_host;
             }
             // try to select host by mail domain
