@@ -68,6 +68,13 @@ class rcube_message_header
      */
     public $cc;
 
+    /**
+     * Message hidden recipients (Bcc)
+     *
+     * @var string
+     */
+    public $bcc;
+
     /**
      * Message Reply-To header
      *
@@ -201,6 +208,28 @@ class rcube_message_header
      */
     public $flags = [];
 
+    /**
+     * Message annotations (RFC 5257)
+     *
+     * @var ?array
+     */
+    public $annotations;
+
+    /**
+     * Extra flags (for the messages list)
+     *
+     * @var array
+     * @deprecated Use $flags
+     */
+    public $list_flags = [];
+
+    /**
+     * Extra columns content (for the messages list)
+     *
+     * @var array
+     */
+    public $list_cols = [];
+
     /**
      * Message structure
      *
@@ -289,7 +318,7 @@ class rcube_message_header
      * @param string $name   Header name
      * @param bool   $decode Decode the header content
      *
-     * @param string|null Header content
+     * @return string|null Header content
      */
     public function get($name, $decode = true)
     {
