@@ -201,7 +201,7 @@ class rcube_vcard
                     $key     = $col;
                     $subtype = '';
 
-                    if (!empty($raw['type'])) {
+                    if (!empty($raw['type']) && is_array($raw['type'])) {
                         $raw['type'] = array_map('strtolower', $raw['type']);
 
                         $combined = implode(',', array_diff($raw['type'], ['internet', 'pref']));
@@ -305,7 +305,7 @@ class rcube_vcard
     /**
      * Convert the data structure into a vcard 3.0 string
      *
-     * @param bool $folder Use RFC2425 folding
+     * @param bool $folded Use RFC2425 folding
      *
      * @return string vCard output
      */
@@ -318,7 +318,7 @@ class rcube_vcard
     /**
      * Clear the given fields in the loaded vcard data
      *
-     * @param array List of field names to be reset
+     * @param array $fields List of field names to be reset
      */
     public function reset($fields = [])
     {
@@ -699,8 +699,8 @@ class rcube_vcard
     private static function vcard_decode($vcard)
     {
         // Perform RFC2425 line unfolding and split lines
-        $vcard  = preg_replace(["/\r/", "/\n\s+/"], '', $vcard);
-        $lines  = explode("\n", $vcard);
+        $vcard = str_replace(["\r", "\n ", "\n\t"], '', $vcard);
+        $lines = explode("\n", $vcard);
         $result = [];
 
         for ($i=0; $i < count($lines); $i++) {
@@ -877,7 +877,17 @@ class rcube_vcard
                         }
                         else {
                             foreach ((array)$attrvalues as $attrvalue) {
-                                $attr .= strtoupper(";$attrname=") . self::vcard_quote($attrvalue, ',');
+                                $attrname = strtoupper($attrname);
+                                // TYPE=OTHER is non-standard, TYPE=INTERNET is redundant, remove these
+                                if ($attrname == 'TYPE') {
+                                    $attrvalue = array_map('strtolower', (array) $attrvalue);
+                                    $attrvalue = array_diff($attrvalue, ['other', 'internet']);
+                                    if (empty($attrvalue)) {
+                                        continue;
+                                    }
+                                }
+
+                                $attr .= ';' . $attrname . '=' . self::vcard_quote($attrvalue, ',');
                             }
                         }
                     }
@@ -1003,8 +1013,8 @@ class rcube_vcard
         // This will for example exclude photos
 
         // Perform RFC2425 line unfolding and split lines
-        $string = preg_replace(["/\r/", "/\n\s+/"], '', $string);
-        $lines  = explode("\n", $string);
+        $string = str_replace(["\r", "\n ", "\n\t"], '', $string);
+        $lines = explode("\n", $string);
         $string = '';
 
         for ($i = 0, $len = count($lines); $i < $len; $i++) {
