@@ -164,7 +164,14 @@ class rcube_washtml
     public $extlinks = false;
 
     /** @var array Current settings */
-    private $config = [];
+    private $config = [
+        'add_comments' => true,
+        'allow_remote' => false,
+        'base_url' => '',
+        'charset' => RCUBE_CHARSET,
+        'cid_map' => [],
+        'show_washed' => true,
+    ];
 
     /** @var array Registered callback functions for tags */
     private $handlers = [];
@@ -225,11 +232,7 @@ class rcube_washtml
 
         unset($p['html_elements'], $p['html_attribs'], $p['ignore_elements'], $p['void_elements'], $p['css_prefix']);
 
-        $this->config = $p + ['show_washed' => true, 'allow_remote' => false, 'cid_map' => [], 'base_url' => ''];
-
-        if (!isset($this->config['charset'])) {
-            $this->config['charset'] = RCUBE_CHARSET;
-        }
+        $this->config = array_merge($this->config, $p);
     }
 
     /**
@@ -254,48 +257,19 @@ class rcube_washtml
     {
         $result = [];
 
-        // Remove unwanted white-space characters so regular expressions below work better
-        $style = preg_replace('/[\n\r\s\t]+/', ' ', $style);
-
         // Decode insecure character sequences
         $style = rcube_utils::xss_entity_decode($style);
 
-        foreach (rcube_utils::parse_css_block($style) as $rule) {
-            $cssid = $rule[0];
-            $value = '';
+        // Remove unwanted white-space characters
+        $style = preg_replace('/[\n\r\t]+/', ' ', $style);
 
-            foreach ($this->explode_style($rule[1]) as $val) {
-                if (preg_match('/^url\(/i', $val)) {
-                    if (preg_match('/^url\(\s*[\'"]?([^\'"\)]*)[\'"]?\s*\)/iu', $val, $match)) {
-                        if ($url = $this->wash_uri($match[1])) {
-                            $value .= ' url(' . htmlspecialchars($url, ENT_QUOTES, $this->config['charset']) . ')';
-                        }
-                    }
-                }
-                else if (!preg_match('/^(behavior|expression)/i', $val)) {
-                    // Set position:fixed to position:absolute for security (#5264)
-                    if (!strcasecmp($cssid, 'position') && !strcasecmp($val, 'fixed')) {
-                        $val = 'absolute';
-                    }
-
-                    // whitelist ?
-                    $value .= ' ' . $val;
-
-                    // #1488535: Fix size units, so width:800 would be changed to width:800px
-                    if (preg_match('/^(left|right|top|bottom|width|height)/i', $cssid)
-                        && preg_match('/^[0-9]+$/', $val)
-                    ) {
-                        $value .= 'px';
-                    }
-                }
+        $uri_callback = function ($uri) {
+            if ($uri = $this->wash_uri($uri)) {
+                return htmlspecialchars($uri, \ENT_QUOTES, $this->config['charset']);
             }
-
-            if (isset($value[0])) {
-                $result[] = $cssid . ': ' . trim($value);
-            }
-        }
-
-        return implode('; ', $result);
+        };
+ 
+        return rtrim(rcube_utils::sanitize_css_block($style, $uri_callback), ';');
     }
 
     /**
@@ -579,7 +553,7 @@ class rcube_washtml
                 );
             }
 
-            return '<!-- ignored -->';
+            return $this->config['add_comments'] ? '<!-- ignored -->' : '';
         }
 
         $node = $node->firstChild;
@@ -593,7 +567,9 @@ class rcube_washtml
                 if ($tagName == 'link') {
                     $uri = $this->wash_uri($node->getAttribute('href'), false, false);
                     if (!$uri) {
-                        $dump .= '<!-- link ignored -->';
+                        if ($this->config['add_comments']) {
+                            $dump .= '<!-- link ignored -->';
+                        }
                         break;
                     }
 
@@ -603,7 +579,9 @@ class rcube_washtml
                     && self::attribute_value($node, 'attributename', 'href')
                 ) {
                     // Insecure svg tags
-                    $dump .= "<!-- $tagName blocked -->";
+                    if ($this->config['add_comments']) {
+                        $dump .= "<!-- {$tagName} blocked -->";
+                    }
                     break;
                 }
 
@@ -614,13 +592,13 @@ class rcube_washtml
                 }
                 else if (isset($this->_html_elements[$tagName])) {
                     $content = $this->dumpHtml($node, $level);
-                    $dump .= '<' . $node->nodeName;
+                    $tag = '<' . $node->nodeName;
 
                     if ($tagName == 'svg') {
                         $xpath = new DOMXPath($node->ownerDocument);
                         foreach ($xpath->query('namespace::*') as $ns) {
                             if ($ns->nodeName != 'xmlns:xml') {
-                                $dump .= sprintf(' %s="%s"',
+                                $tag .= sprintf(' %s="%s"',
                                     $ns->nodeName,
                                     htmlspecialchars($ns->nodeValue, ENT_QUOTES, $this->config['charset'])
                                 );
@@ -631,20 +609,25 @@ class rcube_washtml
                         $content = htmlspecialchars($content, ENT_QUOTES | ENT_SUBSTITUTE, $this->config['charset']);
                     }
 
-                    $dump .= $this->wash_attribs($node);
+                    $tag .= $this->wash_attribs($node);
 
-                    if ($content === '' && ($this->is_xml || isset($this->_void_elements[$tagName]))) {
-                        $dump .= ' />';
-                    }
-                    else {
-                        $dump .= '>' . $content . '</' . $node->nodeName . '>';
+                    if (isset($this->_ignore_elements[$tagName])) {
+                        $dump .= $content;
+                    } elseif ($content === '' && ($this->is_xml || isset($this->_void_elements[$tagName]))) {
+                         $dump .= $tag . ' />';
+                    } else {
+                         $dump .= $tag . '>' . $content . '</' . $node->nodeName . '>';
                     }
                 }
                 else if (isset($this->_ignore_elements[$tagName])) {
-                    $dump .= '<!-- ' . htmlspecialchars($node->nodeName, ENT_QUOTES, $this->config['charset']) . ' not allowed -->';
+                    if ($this->config['add_comments']) {
+                        $dump .= '<!-- ' . htmlspecialchars($node->nodeName, ENT_QUOTES, $this->config['charset']) . ' not allowed -->';
+                    }
                 }
                 else {
-                    $dump .= '<!-- ' . htmlspecialchars($node->nodeName, ENT_QUOTES, $this->config['charset']) . ' ignored -->';
+                    if ($this->config['add_comments']) {
+                        $dump .= '<!-- ' . htmlspecialchars($node->nodeName, ENT_QUOTES, $this->config['charset']) . ' ignored -->';
+                    }
                     $dump .= $this->dumpHtml($node, $level); // ignore tags not its content
                 }
                 break;
@@ -726,6 +709,12 @@ class rcube_washtml
      */
     public function get_config($prop)
     {
+        $config_props = ['html_elements', 'html_attribs', 'ignore_elements', 'void_elements', 'css_prefix'];
+
+        if (in_array($prop, $config_props)) {
+            return $this->{"_{$prop}"};
+        }
+
         return $this->config[$prop] ?? null;
     }
 
@@ -978,39 +967,4 @@ class rcube_washtml
 
         return $html;
     }
-
-    /**
-     * Explode css style value
-     *
-     * @param string $style CSS style
-     *
-     * @return array List of CSS rules
-     */
-    protected function explode_style($style)
-    {
-        $result = [];
-        $strlen = strlen($style);
-        $q      = false;
-
-        // explode value
-        for ($p = $i = 0; $i < $strlen; $i++) {
-            if (($style[$i] == "\"" || $style[$i] == "'") && ($i == 0 || $style[$i-1] != "\\")) {
-                if ($q == $style[$i]) {
-                    $q = false;
-                }
-                else if (!$q) {
-                    $q = $style[$i];
-                }
-            }
-
-            if (!$q && $style[$i] == ' ' && ($i == 0 || !preg_match('/[,\(]/', $style[$i-1]))) {
-                $result[] = substr($style, $p, $i - $p);
-                $p = $i + 1;
-            }
-        }
-
-        $result[] = (string) substr($style, $p);
-
-        return $result;
-    }
 }
